﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;

using DevExpress.Data;
using DevExpress.Xpf.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 언바운드 데이터 리스트
        /// </summary>
        private List<bool> unboundDataList = new List<bool>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            Loaded += Window_Loaded;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.gridControl.ItemsSource = new NorthwindEntities().Products;

            SetUnboundDataSource(this.gridControl.VisibleRowCount);

            this.gridControl.CustomSummary += gridControl_CustomSummary;
        }

        #endregion
        #region 그리드 컨트롤 커스텀 요약 처리하기 - gridControl_CustomSummary(sender, e)

        /// <summary>
        /// 그리드 컨트롤 커스텀 요약 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridControl_CustomSummary(object sender, CustomSummaryEventArgs e)
        {
            if(e.SummaryProcess == CustomSummaryProcess.Start)
            {
                e.TotalValue = 0m;
            }

            if(e.SummaryProcess == CustomSummaryProcess.Calculate)
            {
                if((bool)e.GetValue("Selected"))
                {
                    e.TotalValue = Convert.ToDecimal(e.TotalValue) + Convert.ToDecimal(e.GetValue("UnitPrice"));
                }
            }
        }

        #endregion
        #region 그리드 컨트롤 커스텀 언바운드 컬럼 데이터 처리하기 - gridControl_CustomUnboundColumnData(sender, e)

        /// <summary>
        /// 그리드 컨트롤 커스텀 언바운드 컬럼 데이터 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridControl_CustomUnboundColumnData(object sender, GridColumnDataEventArgs e)
        {
            if(e.Column.FieldName == "Selected")
            {
                if(e.IsGetData)
                {
                    e.Value = this.unboundDataList[e.ListSourceRowIndex];
                }

                if(e.IsSetData)
                {
                    this.unboundDataList[e.ListSourceRowIndex] = (bool)e.Value;
                }
            }
        }

        #endregion
        #region 테이블 뷰 마우스 더블 클릭시 처리하기 - tableView_MouseDoubleClick(sender, e)

        /// <summary>
        /// 테이블 뷰 마우스 더블 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void tableView_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            int rowHandle = this.gridControl.View.GetRowHandleByMouseEventArgs(e);

            if(this.gridControl.IsValidRowHandle(rowHandle))
            {
                this.gridControl.SetCellValue(rowHandle, "Selected", !(bool)this.gridControl.GetCellValue(rowHandle, "Selected"));

                this.gridControl.RefreshData();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 언바운드 데이터 소스 설정하기 - SetUnboundDataSource(count)

        /// <summary>
        /// 언바운드 데이터 소스 설정하기
        /// </summary>
        /// <param name="count">카운트</param>
        private void SetUnboundDataSource(int count)
        {
            for(int i = 0; i < count; i++)
            {
                this.unboundDataList.Add(false);
            }
        }

        #endregion
    }
}